    /*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package analog;

import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import javax.swing.UnsupportedLookAndFeelException;
import w1.PowerMeter;
import w1.Vector2D;

/**
 *
 * @author mm
 */
public class SwrMeter extends javax.swing.JPanel implements PowerMeter {

    private Image meter;
    private boolean peakHold;
    private boolean digitalMode;
    private boolean monochrome;
    private char range;
    private double angStartRfl;
    private double angStartFwd;
    private double angExtent;
    private double rfl_rad;
    private double fwd_rad;
    private double rfl_W, fwd_W, rflHold_rad, fwdHold_rad;
    private double rflDigi_W, fwdDigi_W;
    private double fwdMax, rflMax;
    private double fwdHold_W, rflHold_W; // For peak and hold.
    private int holdCounter, holdCounterMax;
    private int fwdXc, fwdYc; // Center of Forward meter arc.
    private int rflXc, rflYc; // Center of Reflected meter arc.
    private Color rflColor, fwdColor, rflNeedleColor, fwdNeedleColor,
            swrColor, rflFontColor, fwdFontColor;
    public String firmwareVersion;
    static final int TIMEOUT = 60; // milliseconds, timer setting.

    /**
     * Creates new form SwrMeter
     */
    public SwrMeter() {
        initComponents();
        fwdHold_W = rflHold_W = 0.;
        rfl_rad = rflHold_rad = Math.toRadians(10);
        fwd_rad = fwdHold_rad = Math.toRadians(170);
        holdCounter = holdCounterMax = 0;
        peakHold = false;
        angStartRfl = 10;
        angStartFwd = 100;
        angExtent = 70;
        rflMax = 6;
        fwdMax = 15;
        digitalMode = false;
        firmwareVersion = "";
        setMonochrome(false);
        range = '0';
    }

    @Override
    public void startFrameTimer() {
    }

    public boolean isDigitalMode() {
        return digitalMode;
    }

    public void setDigitalMode(boolean digitalMode) {
        this.digitalMode = digitalMode;
    }

    @Override
    public boolean isMonochrome() {
        return monochrome;
    }

    @Override
    public void setMonochrome(boolean monochrome) {
        this.monochrome = monochrome;
        if (monochrome) {
            rflColor = Color.gray;
            fwdColor = Color.gray;
            swrColor = Color.gray;
            rflNeedleColor = Color.black;
            fwdNeedleColor = Color.black;
            fwdFontColor = Color.black;
            rflFontColor = Color.gray;
        } else {
            rflColor = Color.blue;
            fwdColor = Color.black;
            swrColor = Color.red;
            rflNeedleColor = Color.LIGHT_GRAY;
            fwdNeedleColor = Color.gray;
            fwdFontColor = Color.black;
            rflFontColor = Color.blue;
        }
        repaint();
    }

    @Override
    public void stopFrameTimer() {
    }

    @Override
    public void setFirmwareVersion(String firmwareVersion) {
        if (firmwareVersion == null) {
            firmwareVersion = "";
        }
        this.firmwareVersion = firmwareVersion;
        meter = createMeterImage();
    }

    /**
     * Indicates if this meter is currently display peak-hold or peak. Peak-hold
     * duration can be specified with setDecayRate().
     */
    @Override
    public boolean isPeakHold() {
        return peakHold;
    }

    /**
     * Used to specify if meter should display peak-hold or peak.
     */
    @Override
    public void setPeakHold(boolean peakHold) {
        this.peakHold = peakHold;
    }

    @Override
    public void setFwd_W(double fwd_W) {
        this.fwd_W = fwd_W;
    }

    @Override
    public void setRfl_W(double rfl_W) {
        this.rfl_W = rfl_W;
        setPower_W(fwd_W, rfl_W);
    }

    @Override
    public boolean isLedsOn() {
        return false;
    }

    @Override
    public void setLedsOn(boolean LEDsOn) {
    }

    @Override
    public boolean isPEP() {
        return false;
    }

    @Override
    public void setPEP(boolean pep) {
    }

    public void setPower_W(double fwd_W, double rfl_W) {

        fwdDigi_W = fwd_W; // Need unscaled power for digital display.
        rflDigi_W = rfl_W;

        // Adjust scale.
        switch (range) {
            case '0':
                if (fwd_W < 1.4) {
                    fwd_W *= 10;
                    rfl_W *= 10;
                } else if (fwd_W > 14) {
                    fwd_W /= 10;
                    rfl_W /= 10;
                }
                break;
            case '1': // Max 1.4 W
                fwd_W *= 10;
                rfl_W *= 10;
                break;
            case '2': // Max 14 W
                // No need for changes.
                break;
            case '3': // Max 140 W
                fwd_W /= 10;
                rfl_W /= 10;
                break;
            default:
                System.out.println(getClass() + ": setPower_W: hould never be here!");
                break;
        }

        fwd_W = cap(0, fwd_W, fwdMax);
        rfl_W = cap(0, rfl_W, rflMax);

        this.fwd_W = fwd_W; // Need scaled power for analog display.
        this.rfl_W = rfl_W;

        rfl_rad = Math.toRadians(angStartRfl)
                + scaleAngle(angExtent, rflMax, rfl_W + 1);
        fwd_rad = Math.toRadians(angStartFwd + angExtent)
                - scaleAngle(angExtent, fwdMax, fwd_W + 1);

        // Take care of peak-hold decay.
        if (peakHold) {
            holdCounter--;
            if (holdCounter <= 0 || fwdHold_W < fwd_W
                    || rflHold_W < rfl_W) {

                // Reset hold counter.
                holdCounter = holdCounterMax;
                rflHold_rad = rfl_rad;
                fwdHold_rad = fwd_rad;
            }
        }
        repaint();
    }

    private double cap(double min, double val, double max) {

        val = (val < min) ? min : val;
        val = (val > max) ? max : val;
        return val;
    }

    /**
     * Set the decay rate of peak-hold measurements. The specified "rate" is
     * actually a counter which is decremented at each call to setMeasurement().
     */
    @Override
    public void setDecayRate(char rate) {
        switch (rate) {
            case 'S': // slow
                holdCounterMax = 12;
                break;
            case 'M': // medium
                holdCounterMax = 6;
                break;
            case 'F': // fast
                holdCounterMax = 3;
                break;
            default:
                holdCounterMax = 1;
                break;
        }
    }

    @Override
    public void setDropRate(char dropRate) {
    }

    @Override
    public void setRange(char range) {
        this.range = range;
    }

    public void recreateMeterImage() {
        meter = null;
    }

    /**
     * Create an image of an analog SWR meter.
     *
     * @return the newly created image.
     */
    private Image createMeterImage() {

        int width = getWidth();
        int height = getHeight();
        int radius = (int) (0.98 * height);
        BufferedImage bi = new BufferedImage(
                width, height,
                BufferedImage.TYPE_4BYTE_ABGR);

        // Make nice smooth and anti-aliased lines.
        Graphics2D g2 = bi.createGraphics();
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setRenderingHint(RenderingHints.KEY_RENDERING,
                RenderingHints.VALUE_RENDER_QUALITY);
        g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
                RenderingHints.VALUE_STROKE_PURE);

        // Give meter a white background.
        g2.setColor(Color.white);
        g2.fillRect(0, 0, width, height);

        g2.setStroke(new BasicStroke(5));

        // Position meters.
        int offsetRflX = (int) (-0.5 * radius);
        int offsetY = (int) (0.08 * radius);
        rflXc = (int) (offsetRflX + radius);
        rflYc = (int) (offsetY + radius);

        double offsetFwdX = (int) (0.1 * width);
        fwdXc = (int) (offsetFwdX + radius);
        fwdYc = (int) (offsetY + radius);

        // Draw reflected power arc.
        g2.setColor(rflColor);

        Arc2D rflArc = new Arc2D.Double();
        rflArc.setArc(offsetRflX, offsetY, 2 * radius, 2 * radius,
                angStartRfl, angExtent, Arc2D.OPEN);
        g2.draw(rflArc);

        // Draw tic marks logarithmically on reflection axis.
        for (int i = 1; i <= rflMax; i++) {
            Line2D tic = mkTicRfl(radius, offsetRflX, offsetY, angStartRfl,
                    angExtent, i);
            g2.setColor(rflColor);
            g2.draw(tic);
            // Label tic mark.
            String label = String.format("%d", (int) (i - 1));
            int fontStyle = monochrome ? Font.ITALIC : Font.PLAIN;
            g2.setColor(rflFontColor);
            labelTic(g2, label, tic, fontStyle);
        }

        // Label arc.
        g2.setColor(rflFontColor);
        String label = "Reflected (W)";
        arcText(g2, label, rflXc, rflYc, radius, angStartRfl, angExtent);

        // Draw minor tic marks on reflected power axis.
        g2.setColor(rflColor);
        g2.setStroke(new BasicStroke(1f));
        g2.draw(mkTicRfl(radius, offsetRflX, offsetY, angStartRfl, angExtent,
                1.2));
        g2.draw(mkTicRfl(radius, offsetRflX, offsetY, angStartRfl, angExtent,
                1.4));
        g2.draw(mkTicRfl(radius, offsetRflX, offsetY, angStartRfl, angExtent,
                1.6));
        g2.draw(mkTicRfl(radius, offsetRflX, offsetY, angStartRfl, angExtent,
                1.8));
        g2.draw(mkTicRfl(radius, offsetRflX, offsetY, angStartRfl, angExtent,
                2.5));
        g2.draw(mkTicRfl(radius, offsetRflX, offsetY, angStartRfl, angExtent,
                3.5));

        // Draw forward power arc.
        g2.setStroke(new BasicStroke(5));
        g2.setColor(fwdColor);
        Arc2D fwdArc = new Arc2D.Double();
        fwdArc.setArc(offsetFwdX, offsetY, 2 * radius, 2 * radius,
                angStartFwd, angExtent, Arc2D.OPEN);
        g2.draw(fwdArc);

        // Draw major tic marks on forward power axis.
        for (int i = 1; i <= fwdMax; i++) {
            Line2D tic = mkTicFwd(radius, offsetFwdX, offsetY, angStartFwd,
                    angExtent, i);
            g2.setColor(fwdColor);
            g2.draw(tic);
            // Label tic mark.
            if (i < 8 || i % 2 != 0) {
                label = String.format("%d", (int) (i - 1));
                g2.setColor(fwdFontColor);
                labelTic(g2, label, tic, Font.PLAIN);
            }
        }
        // Draw minor tic marks on forward power axis.
        g2.setColor(fwdColor);
        g2.setStroke(new BasicStroke(1));
        g2.draw(mkTicFwd(radius, offsetFwdX, offsetY, angStartFwd, angExtent,
                1.5));
        g2.draw(mkTicFwd(radius, offsetFwdX, offsetY, angStartFwd, angExtent,
                2.5));

        // Label arc.
        label = "Forward (W)";
        g2.setColor(fwdFontColor);
        arcText(g2, label, fwdXc, fwdYc, radius, angStartFwd, angExtent);

        // Draw SWR curves in red.
        g2.setColor(swrColor);
        g2.setStroke(new BasicStroke(1.5f));

        Point2D p0 = new Point2D.Double();
        Point2D p1 = new Point2D.Double();

        Ellipse2D clip = new Ellipse2D.Double(width * 3. / 18, height / 3.3,
                11. / 18 * width, 1.4 * height);
//        g2.draw(clip);
        g2.setClip(clip);

        g2.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
        double[] swrs = {1, 1.1, 1.2, 1.3, 1.4, 1.5, 1.8, 2, 3, 5};
        for (double swr : swrs) {
            Line2D l = new Line2D.Double();
            boolean first = true;
            for (double fwd_W = 0; fwd_W <= fwdMax; fwd_W += 0.05) {

                // Find reflected power base on forward and SWR.
                double rfl_W = calcRfl_W(fwd_W, swr);

                double rflAngle_rad = Math.toRadians(angStartRfl)
                        + scaleAngle(angExtent, rflMax, rfl_W + 1);
                double fwdAngle_rad = Math.toRadians(angStartFwd + angExtent)
                        - scaleAngle(angExtent, fwdMax, fwd_W + 1);

                Line2D rflLine = new Line2D.Double(rflXc, rflYc,
                        rflXc + radius * Math.cos(rflAngle_rad),
                        rflYc - radius * Math.sin(rflAngle_rad));
                Line2D fwdLine = new Line2D.Double(fwdXc, fwdYc,
                        fwdXc + radius * Math.cos(fwdAngle_rad),
                        fwdYc - radius * Math.sin(fwdAngle_rad));

                p0.setLocation(p1);
                p1.setLocation(linesIntersection(rflLine, fwdLine));
                if (first) {
                    // First time through, don't yet have 2 endpoints to draw.
                    first = false;
                } else {
                    l.setLine(p0, p1);
                    g2.draw(l);
                }
            }
            // Extend line segment.
            double x1 = l.getX1();
            double y1 = l.getY1();
            double x2 = l.getX2();
            double y2 = l.getY2();
            double m = (y2 - y1) / (x2 - x1);
            x2 = x1 + 100 * radius * (x2 - x1);
            y2 = y2 + 100 * radius * (y2 - y1);
            l.setLine(x1, y1, x2, y2);
            g2.draw(l);
        }
        g2.setClip(null);
        g2.setColor(Color.white);
        Ellipse2D e = new Ellipse2D.Double(width / 2 - 30, height - 20, 60, 60);
        g2.fill(e);

        // Label line with SWR ratio.
        g2.setColor(swrColor);
        Point2D.Float loc = mkSwrLoc(0.8 * radius, rflXc, rflYc, angStartRfl,
                angExtent, 0.98);
        g2.drawString("1", loc.x, loc.y);
        loc = mkSwrLoc(0.79 * radius, rflXc, rflYc, angStartRfl, angExtent, 1.1);
        g2.drawString("1.1", loc.x, loc.y);
        loc = mkSwrLoc(0.8 * radius, rflXc, rflYc, angStartRfl, angExtent, 1.31);
        g2.drawString("1.2", loc.x, loc.y);
        loc =
                mkSwrLoc(0.81 * radius, rflXc, rflYc, angStartRfl, angExtent,
                1.52);
        g2.drawString("1.3", loc.x, loc.y);
        loc =
                mkSwrLoc(0.815 * radius, rflXc, rflYc, angStartRfl, angExtent,
                1.7);
        g2.drawString("1.4", loc.x, loc.y);
        loc = mkSwrLoc(0.83 * radius, rflXc, rflYc, angStartRfl, angExtent, 1.9);
        g2.drawString("1.5", loc.x, loc.y);
        loc = mkSwrLoc(0.835 * radius, rflXc, rflYc, angStartRfl, angExtent,
                2.32);
        g2.drawString("1.8", loc.x, loc.y);
        loc =
                mkSwrLoc(0.85 * radius, rflXc, rflYc, angStartRfl, angExtent,
                2.72);
        g2.drawString("2", loc.x, loc.y);
        loc =
                mkSwrLoc(0.85 * radius, rflXc, rflYc, angStartRfl, angExtent,
                3.64);
        g2.drawString("3", loc.x, loc.y);
        loc = mkSwrLoc(0.83 * radius, rflXc, rflYc, angStartRfl, angExtent, 4.5);
        g2.drawString("5", loc.x, loc.y);

        g2.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
        g2.setColor(new Color(160, 160, 160)); // Light gray.
        FontMetrics fm = g2.getFontMetrics();

        // Print firmware version.
        int sx = fm.stringWidth(firmwareVersion);
        int sy = height - 3 * fm.getHeight();
        g2.drawString(firmwareVersion, width / 2 - sx, sy + fm.getHeight());

        // Print AB3AP.
        String call = "AB3AP";
        int ax = (sx - fm.stringWidth(call));
        g2.drawString(call, width / 2 - sx + ax, sy);

        return bi;
    }

    private void arcText(Graphics2D g2, String s,
            double ctrX, double ctrY, double radius,
            double angStart_deg, double angExtent_deg) {

        Font font = new Font(Font.SANS_SERIF, Font.PLAIN, 20);
        g2.setFont(font);
        int len = s.length();

        FontRenderContext frc = g2.getFontRenderContext();
        GlyphVector gv = font.createGlyphVector(frc, s);
        double totLenGlyphs = gv.getLogicalBounds().getWidth();

        for (int i = 0; i < len; i++) {
            double xGlyph = gv.getGlyphPosition(i).getX();
            double relPos = xGlyph / totLenGlyphs;
            double start_deg = angStart_deg + angExtent_deg - 22;
            double offset_deg = relPos * (angExtent_deg - 45);
            double theta_rad = Math.toRadians(start_deg - offset_deg);
            double dx = ctrX + 1.15 * radius * Math.cos(theta_rad);
            double dy = ctrY - 1.15 * radius * Math.sin(theta_rad);

//            at.rotate(theta_rad, at.getTranslateX(),  at.getTranslateY());
            g2.translate(dx, dy);
            g2.rotate(Math.PI / 2 - theta_rad);
            g2.drawString(String.valueOf(s.charAt(i)), 0f, 0f);
            g2.rotate(theta_rad - Math.PI / 2);
            g2.translate(-dx, -dy);
        }
    }

    private Point2D.Float mkSwrLoc(double radius, double cX, double cY,
            double angStart, double angExtent, double val) {

        // Calculate fraction of angular extent is this tic mark.
        double angle_rad = scaleAngle(angExtent, rflMax, val);
        angle_rad += Math.toRadians(angStart);
        double x0 = radius * Math.cos(angle_rad);
        double y0 = radius * Math.sin(angle_rad);
        x0 += cX;
        y0 = cY - y0;
        return new Point2D.Float((float) x0, (float) y0);
    }

    private Line2D mkTicRfl(double radius, double offsetX, double offsetY,
            double angStart, double angExtent, double val) {

        // Calculate fraction of angular extent is this tic mark.
        double angle_rad = scaleAngle(angExtent, rflMax, val);
        angle_rad += Math.toRadians(angStart);
        return mkTic(radius, offsetX, offsetY, angle_rad, val);
    }

    private Line2D mkTicFwd(double radius, double offsetX, double offsetY,
            double angStart, double angExtent, double val) {

        // Calculate fraction of angular extent is this tic mark.
        double angle_rad = scaleAngle(angExtent, fwdMax, val);
        angle_rad = Math.toRadians(angStart + angExtent) - angle_rad;
        return mkTic(radius, offsetX, offsetY, angle_rad, val);
    }

    private Line2D mkTic(double radius, double offsetX,
            double offsetY, double angle_rad, double val) {

        // Calculate tic mark endpoints.
        double x0 = radius * Math.cos(angle_rad);
        double y0 = radius * Math.sin(angle_rad);
        double x1 = (radius + 10) * Math.cos(angle_rad);
        double y1 = (radius + 10) * Math.sin(angle_rad);
        x0 += offsetX + radius;
        x1 += offsetX + radius;
        y0 = radius - (y0 - offsetY);
        y1 = radius - (y1 - offsetY);
        Line2D.Double l = new Line2D.Double(x0, y0, x1, y1);
        return l;
    }

    private void labelTic(Graphics2D g2, String label, Line2D tic,
            int fontStyle) {

        double deltaY = tic.getY2() - tic.getY1();
        double deltaX = tic.getX2() - tic.getX1();
        double m = deltaY / deltaX;

        Font font = new Font(Font.SANS_SERIF, fontStyle, 20);
        g2.setFont(font);
        FontMetrics fm = g2.getFontMetrics();
        int swidth = fm.stringWidth(label);
        int sheight = fm.getHeight();

        double d = 13; // Pixels between text and tic.
        // Find coords of center of label bounding box.
//        double a = d * Math.cos(theta);
//        double ctrX = tic.getX2() + a;
//        ctrX += ((a > 0) ? 1 : -1) * swidth / 2.;
//        double ctrY = tic.getY2() - a * Math.atan(theta);

        Vector2D v = new Vector2D(tic.getX1(), tic.getY1(),
                tic.getX2(), tic.getY2());
        v = v.normalize();
        v = v.scale(d);
        v = v.add(tic.getX2(), tic.getY2());

        double ctrX = v.getX();
        double ctrY = v.getY();

        float x0 = (float) (ctrX - swidth / 2.);
        float y0 = (float) (ctrY + sheight / 4);

        g2.drawString(label, x0, y0);
    }

    /**
     * From http://en.wikipedia.org/wiki/Line-line_intersection
     *
     * @param l1
     * @param l2
     * @return
     */
    private Point2D linesIntersection(Line2D l1, Line2D l2) {

        double x1 = l1.getX1();
        double y1 = l1.getY1();
        double x2 = l1.getX2();
        double y2 = l1.getY2();

        double x3 = l2.getX1();
        double y3 = l2.getY1();
        double x4 = l2.getX2();
        double y4 = l2.getY2();

        double xNum = (x1 * y2 - y1 * x2) * (x3 - x4)
                - (x1 - x2) * (x3 * y4 - y3 * x4);
        // If den==0, lines are parallel.
        double den = (x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4);

        double yNum = (x1 * y2 - y1 * x2) * (y3 - y4)
                - (y1 - y2) * (x3 * y4 - y3 * x4);

        return new Point2D.Double(xNum / den, yNum / den);
    }

    /**
     * Find third point in this triangle:
     * <pre>
     *      /--
     *     /    ---
     *    /        ---
     *   /            ----
     *  /                  ---
     * / left            right ---
     * ------------------------------
     *            base
     * </pre>
     *
     * @param base
     * @param left_rad
     * @param right_rad
     * @return
     */
    private Point2D findThirdPoint(
            Line2D base, double left_rad, double right_rad) {

        // Find angle opposite base.
        double base_rad = Math.PI - left_rad - right_rad;

        // Find length of base.
        double xlen = base.getX1() - base.getX2();
        double ylen = base.getY1() - base.getY2();
        double baseLen = Math.sqrt(xlen * xlen + ylen * ylen);

        // Use Law of Sines to find length of side opposite right_rad.
        double rightLen = baseLen / base_rad * right_rad;

        // Use trig to find point of intersection.
        double isctX = base.getX1() + rightLen * Math.cos(left_rad);
        double isctY = base.getY1() - rightLen * Math.sin(left_rad);

        return new Point2D.Double(isctX, isctY);
    }

    /**
     * Find angle on arc scale corresponding to a given value. Scale's minVal
     * assumed to be 1 (log10(1) = 0).
     *
     * @param angExtent
     * @param maxVal
     * @param val
     * @return
     */
    private double scaleAngle(double angExtent, double maxVal, double val) {

        double ratio = Math.log10(val) / Math.log10(maxVal);
        double angle_deg = ratio * angExtent;
        return Math.toRadians(angle_deg);
    }

    private double calcRfl_W(double fwd_W, double swr) {

        // swr = (1 + g) / (1 - g), where g = sqrt(rfl_W / fwd_W)
        // so, g = (swr - 1) / (swr + 1)
        // for given swr and fwd_W, 
        //   rfl_W = [(swr - 1) / (swr + 1)]^2 x fwd_W

        double g = (swr - 1) / (swr + 1);
        return g * g * fwd_W;
    }

    private double calcSwr(double fwd_W, double rfl_W) {

        if (rfl_W == 0) {
            return 1;
        }
        double g = Math.sqrt(rfl_W / fwd_W);
        return (1 + g) / (1 - g);
    }

    @Override
    public void paintComponent(Graphics g) {

        Graphics2D g2 = (Graphics2D) g;
        if (meter == null) {
            meter = createMeterImage();
        }
        g.drawImage(meter, 0, 0, this);
        // Make nice smooth and anti-aliased lines.
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setRenderingHint(RenderingHints.KEY_RENDERING,
                RenderingHints.VALUE_RENDER_QUALITY);
        g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
                RenderingHints.VALUE_STROKE_PURE);
        g2.setStroke(new BasicStroke(5, BasicStroke.CAP_ROUND,
                BasicStroke.JOIN_ROUND));

        double radius = 0.98 * getHeight();

        // Draw peak hold needles.
        if (peakHold) {
            g2.setColor(new Color(233, 233, 233));
            Line2D rflLine = new Line2D.Double(rflXc, rflYc,
                    rflXc + radius * Math.cos(rflHold_rad),
                    rflYc - radius * Math.sin(rflHold_rad));
            g2.draw(rflLine);
            g2.setColor(new Color(222, 222, 222));
            Line2D fwdLine = new Line2D.Double(fwdXc, fwdYc,
                    fwdXc + radius * Math.cos(fwdHold_rad),
                    fwdYc - radius * Math.sin(fwdHold_rad));
            g2.draw(fwdLine);
        }

        // Draw reflected power needle.
        g2.setColor(rflNeedleColor);
        double toRflX = rflXc + radius * Math.cos(rfl_rad);
        double toRflY = rflYc - radius * Math.sin(rfl_rad);
        Line2D rflLine = new Line2D.Double(rflXc, rflYc, toRflX, toRflY);
        g2.draw(rflLine);

        Ellipse2D e = new Ellipse2D.Float();
        if (digitalMode) {
            // Draw digital power circle.
            float pwrRadius = 20f;
            Vector2D v = new Vector2D(rflXc, rflYc, toRflX, toRflY);
            double mag = v.magnitude();
            v = v.normalize();
            v = v.scale(0.92 * mag);
            v = v.add(rflXc, rflYc);
            e.setFrame((float) v.getX() - pwrRadius,
                    (float) v.getY() - pwrRadius, 2 * pwrRadius, 2 * pwrRadius);
            g2.setColor(Color.WHITE);
            g2.fill(e);
            g2.setColor(rflNeedleColor);
            g2.draw(e);
            FontMetrics fm = g2.getFontMetrics();
            g2.setColor(Color.BLACK);
            String pwr = formatPower(rflDigi_W);
            int strWidth = fm.stringWidth(pwr);
            int strHeight = fm.getHeight();
            g.drawString(pwr, (int) v.getX() - strWidth / 2,
                    (int) v.getY() + strHeight / 3);
        }

        // Draw forward power needle.
        g2.setColor(fwdNeedleColor);
        double toFwdX = fwdXc + radius * Math.cos(fwd_rad);
        double toFwdY = fwdYc - radius * Math.sin(fwd_rad);
        Line2D fwdLine = new Line2D.Double(fwdXc, fwdYc, toFwdX, toFwdY);
        g2.draw(fwdLine);

        // Find point where needles intersect.
        Line2D l1 = new Line2D.Double(rflXc, rflYc, toRflX, toRflY);
        Line2D l2 = new Line2D.Double(fwdXc, fwdYc, toFwdX, toFwdY);
        Point2D cross = linesIntersection(l1, l2);

        if (digitalMode) {
            // Draw digital power circle.
            float pwrRadius = 20f;
            Vector2D v = new Vector2D(fwdXc, fwdYc, toFwdX, toFwdY);
            double mag = v.magnitude();
            v = v.normalize();
            v = v.scale(0.92 * mag);
            v = v.add(fwdXc, fwdYc);
            e.setFrame((float) v.getX() - pwrRadius,
                    (float) v.getY() - pwrRadius, 2 * pwrRadius, 2 * pwrRadius);
            g2.setColor(Color.WHITE);
            g2.fill(e);
            g2.setColor(fwdNeedleColor);
            g2.draw(e);
            FontMetrics fm = g2.getFontMetrics();
            g2.setColor(Color.BLACK);
            String pwr = formatPower(fwdDigi_W);
            int strWidth = fm.stringWidth(pwr);
            int strHeight = fm.getHeight();
            g.drawString(pwr, (int) v.getX() - strWidth / 2,
                    (int) v.getY() + strHeight / 3);

            // Draw SWR circle.
            e.setFrame((float) cross.getX() - pwrRadius,
                    (float) cross.getY() - pwrRadius,
                    2 * pwrRadius, 2 * pwrRadius);
            g2.setColor(Color.WHITE);
            g2.fill(e);
            g2.setColor(swrColor);
            g2.draw(e);
            String swr = String.format("%.1f:1", calcSwr(fwd_W, rfl_W));
            strWidth = fm.stringWidth(swr);
            strHeight = fm.getHeight();
            g.drawString(swr, (int) cross.getX() - strWidth / 2,
                    (int) cross.getY() + strHeight / 3);
        } else {
            // Draw black dot at point of intersection.
            g2.setColor(Color.BLACK);
            g.fillOval((int) cross.getX(), (int) cross.getY(), 3, 3);
        }
    }

    private String formatPower(double power) {

        if (power < 10) {
            return String.format("%.2f", power);
        } else if (power < 100) {
            return String.format("%.1f", power);
        } else {
            return String.format("%d", Integer.valueOf((int) (0.5 + power)));
        }
    }

    public static void main(String args[]) {

        /*
         * Set the Nimbus look and feel
         */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /*
         * If Nimbus (introduced in Java SE 6) is not available, stay with the
         * default look and feel. For details see
         * http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html
         */
        for (javax.swing.UIManager.LookAndFeelInfo info :
                javax.swing.UIManager.getInstalledLookAndFeels()) {
            if ("Nimbus".equals(info.getName())) {
                try {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                } catch (ClassNotFoundException ex) {
                    Logger.getLogger(SwrMeter.class.getName()).log(Level.SEVERE,
                            null, ex);
                } catch (InstantiationException ex) {
                    Logger.getLogger(SwrMeter.class.getName()).log(Level.SEVERE,
                            null, ex);
                } catch (IllegalAccessException ex) {
                    Logger.getLogger(SwrMeter.class.getName()).log(Level.SEVERE,
                            null, ex);
                } catch (UnsupportedLookAndFeelException ex) {
                    Logger.getLogger(SwrMeter.class.getName()).log(Level.SEVERE,
                            null, ex);
                }
                break;
            }
        }
        //</editor-fold>

        SwrMeter sm = new SwrMeter();
        JFrame f = new JFrame("SWR Meter");
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.add(sm);
        f.pack();
        f.setVisible(true);
        f = null;

//        /*
//         * Create and display the form
//         */
//        java.awt.EventQueue.invokeLater(new Runnable() {
//
//            @Override
//            public void run() {
//            }
//        });
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setFocusable(false);
        setMaximumSize(new java.awt.Dimension(766, 476));
        setMinimumSize(new java.awt.Dimension(766, 476));
        setRequestFocusEnabled(false);
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                resizeHandler(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 766, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 476, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    private void resizeHandler(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_resizeHandler
        meter = null;
        int width = getWidth();
        setSize(width, (int) (width / 1.6));
    }//GEN-LAST:event_resizeHandler
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

    @Override
    public boolean isUnits_W() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setUnits_W(boolean units_W) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
