package digital;
import java.awt.Rectangle;

/** Implement the Decay/LED button on the front of the virtual W1.  The
 * implementation cycles through the bar graph decay speeds slow, medium, and
 * fast and the secondary function toggles LEDs.
 *
 * Mike Markowski<br>
 * March 2007
 */

public class ButtonDecay extends Button {

	/** Create a decay/led button specifying internally used button name (not
	 * displayed on-screen) and the geometry in pixels specifying where the
	 * button is on-screen. */
	ButtonDecay(String bName, int x0, int y0, int xlen, int ylen) {
		name = bName;
		geometry = new Rectangle(x0, y0, xlen, ylen);
		primary = 'M'; // Medium bar graph decay speed.
		secondary = 'L'; // LEDs on.
		pstr = "MED";
		sstr = "";
	}

	/** For each tap, i.e., call to this method, the ButtonDecay cycles
	 * through slow, medium, fast speeds of bar graph decay.  The appropriate
	 * W1 single letter command is returned to put the W1 in the requested
	 * decay state. */
    @Override
	public String getTapFn() {
		switch (primary) {
		case 'S':
			primary = 'M';
			pstr = "MED";
			return "5";
		case 'M':
			primary = 'F';
			pstr = "FAST";
			return "6";
		case 'F':
			primary = 'S';
			pstr = "SLO";
			return "4";
		}
		return null;
	}

	/** This method simply returns the W1 command that toggles LEDs. */
    @Override
	public String getHoldFn() {
		return "L";
	}

}
