/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package w1;

import analog.AnalogPanel;
import digital.DigitalPanel;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;
import tiny.MiniMeter;
import util.Serial;

/**
 *
 * @author mm
 */
public class W1 extends javax.swing.JFrame {

    static enum meters {

        NONE, W1, W2, KAT500
    };
    static final int TIMEOUT = 120; // milliseconds, timer setting.
    private final boolean autoConnect = false;
    private double pwrFwd, pwrRfl, swr;
    private meters meter;
    private int curFace;
    private final AnalogPanel analogPanel;
    private final DigitalPanel digitalPanel;
    private final MiniMeter miniMeter;
    private PowerMeter powerMeter;
    private Timer holdTimer;
    private String fw;
    private final Serial sport;

    /**
     * Creates new form W1
     */
    public W1() {
        initComponents();
        meter = meters.NONE;

        analogPanel = new AnalogPanel(this);
        analogPanel.setInheritsPopupMenu(true);
        digitalPanel = new DigitalPanel(this);
        digitalPanel.setInheritsPopupMenu(true);
        miniMeter = new MiniMeter();
        miniMeter.setInheritsPopupMenu(true);

        powerMeter = digitalPanel;
        powerMeter.startFrameTimer();
        addPanel((JPanel) powerMeter);
        // Get the size of the default screen
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        // Center panel on screen.
        setLocation(((int) dim.getWidth() - getWidth()) / 2,
                ((int) dim.getHeight() - getHeight()) / 2);
        powerMeter.startFrameTimer();

        sport = new Serial();
        if (autoConnect) {
            if (!sport.findW1()) {
                JOptionPane.showMessageDialog(this,
                        "Cannot find W1 on any serial ports.",
                        "W1 discovery error",
                        JOptionPane.ERROR_MESSAGE);
                System.exit(1);
            }
            mkCommandTimer();
            meterInit();
        }
    }

    private void mkCommandTimer() {
        if (holdTimer != null && holdTimer.isRunning()) {
            holdTimer.stop();
        }
        holdTimer = new Timer(TIMEOUT, new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent evt) {
                switch (meter) {
                    case W1:
                        System.out.println("W1");
                        w1cmd("F"); // Set pwrFwd with value with from W1.
                        powerMeter.setFwd_W(pwrFwd); // Update vu meter.
                        w1cmd("R"); // Set pwrRfl with value with from W1.
                        powerMeter.setRfl_W(pwrRfl); // Update vu meter.
                        break;
                    case W2:
                        w2cmd("F"); // Set pwrFwd with value with from W1.
                        powerMeter.setFwd_W(pwrFwd); // Update vu meter.
                        w2cmd("R"); // Set pwrRfl with value with from W1.
                        powerMeter.setRfl_W(pwrRfl); // Update vu meter.
                        break;
                    case KAT500:
                        System.out.println("kat500!!");
                        break;
                    default:
                        System.out.println("default!!");
                        break;
                }
            }
        });
        holdTimer.setRepeats(true);
        holdTimer.start();
    }

    private void meterInit() {
        switch (meter) {
            case W1:
                // U: recall EEPROM settings,
                // L: toggle LEDs on W1 board,
                // 0: set to auto-range,
                // V: get firmware version,
                // P: toggle peak/hold.  Sent twice so response indicates state.
                w1cmd("X0VLLPP");
                break;
            case W2:
                break;
            case KAT500:
                break;
        }
    }

    public JPopupMenu mkPopUpMenu() {

        JPopupMenu pu = new JPopupMenu();

        if (!autoConnect) {
            String[] ports = sport.getSerialPortNames();
            JMenuItem mi = pu.add(" Connect to Port");
            mi.setEnabled(false);
            pu.addSeparator();
            JCheckBoxMenuItem cb = new JCheckBoxMenuItem("W1");
            cb.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent ae) {
                    meter = meters.W1;
                    mkCommandTimer();
                }
            });
            pu.add(cb);
            cb = new JCheckBoxMenuItem("W2");
            cb.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent ae) {
                    meter = meters.W2;
                    mkCommandTimer();
                }
            });
            pu.add(cb);
            cb = new JCheckBoxMenuItem("KAT500");
            cb.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent ae) {
                    meter = meters.KAT500;
                    mkCommandTimer();
                }
            });
            pu.add(cb);
            pu.addSeparator();
            for (String port : ports) {
                mi = pu.add(port);
                mi.addActionListener(new ActionListener() {

                    @Override
                    public void actionPerformed(ActionEvent e) {
                        JMenuItem item = (JMenuItem) e.getSource();
                        String portName = item.getText();
                        if (sport.setSerialPort("W1", portName)) {
                            sport.close();
                            meterInit();
                        } else {
                            JOptionPane.showMessageDialog((JPanel) powerMeter,
                                    "Cannot find W1 on serial port" + portName
                                    + ".",
                                    "W1 discovery error",
                                    JOptionPane.ERROR_MESSAGE);
                        }
                    }
                });
            }
            pu.addSeparator();
        }
        JMenuItem mi = pu.add("   Face");
        mi.setEnabled(false);
        pu.addSeparator();
        mi = pu.add("Analog");
        mi.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (powerMeter == analogPanel) {
                    return;
                }
                digitalPanel.stopFrameTimer();
                bigPanel.remove(digitalPanel);
                bigPanel.remove(miniMeter);
                powerMeter = analogPanel;
                addPanel(analogPanel);
                analogPanel.startFrameTimer();
                mkCommandTimer();
                meterInit();
            }
        });
        mi = pu.add("Digital");
        mi.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (powerMeter == digitalPanel) {
                    return;
                }
                analogPanel.stopFrameTimer();
                bigPanel.remove(analogPanel);
                bigPanel.remove(miniMeter);
                powerMeter = digitalPanel;
                addPanel(digitalPanel);
                digitalPanel.startFrameTimer();
                mkCommandTimer();
                meterInit();
            }
        });
        mi = pu.add("Tiny");
        mi.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (powerMeter == miniMeter) {
                    return;
                }
                analogPanel.stopFrameTimer();
                bigPanel.remove(analogPanel);
                digitalPanel.stopFrameTimer();
                bigPanel.remove(digitalPanel);
                powerMeter = miniMeter;
                addPanel(miniMeter);
                mkCommandTimer();
                meterInit();
            }
        });
        if (powerMeter == analogPanel) {
            pu.addSeparator();
            mi = pu.add("  Color");
            pu.addSeparator();
            mi.setEnabled(false);
            pu.addSeparator();
            mi = pu.add(powerMeter.isMonochrome() ? "Color" : "Monochrome");
            mi.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {
                    JMenuItem mi = (JMenuItem) e.getSource();
                    powerMeter.setMonochrome(mi.getText().equals("Monochrome"));
                }
            });
        } else {
            pu.addSeparator();
            mi = pu.add("   Units");
            mi.setEnabled(false);
            pu.addSeparator();
            mi = pu.add(powerMeter.isUnits_W() ? "dBm" : "W");
            mi.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {
                    powerMeter.setUnits_W(!powerMeter.isUnits_W());
                }
            });
        }
        pu.addSeparator();
        pu.addSeparator();
        mi = pu.add("   Quit");
        mi.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });
        return pu;
    }

    private void addPanel(JPanel panel) {
        Dimension d = panel.getPreferredSize();
        bigPanel.setPreferredSize(d);
        GroupLayout bigPanelLayout = new GroupLayout(bigPanel);
        bigPanel.setLayout(bigPanelLayout);
        bigPanelLayout.setHorizontalGroup(
                bigPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(GroupLayout.Alignment.TRAILING, bigPanelLayout.createSequentialGroup().addGap(0, 0, Short.MAX_VALUE).addComponent(panel, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)));
        bigPanelLayout.setVerticalGroup(
                bigPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(GroupLayout.Alignment.TRAILING, bigPanelLayout.createSequentialGroup().addGap(0, 0, Short.MAX_VALUE).addComponent(panel, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)));
        validate();
        pack();
    }

    /**
     * Send a W1 command over the serial link and pause, typically 25 ms, to
     * allow W1 to respond. Response is then parsed and acted upon by setting
     * the software state to match state of the real W1. NB: for auto-range
     * mode, the updated range is chosen when new readings of both forward and
     * reflected power have been taken. Therefore, you'll want to take the
     * readings as close together as possible, e.g., with a w1cmd("FR") call.
     */
    public void w1cmd(String s) {
        if (s == null) {
            return;
        }
        byte cmd[] = new byte[1];
        byte[] response = new byte[8];
        for (int i = 0; i < s.length(); i++) {
            cmd[0] = (byte) s.charAt(i);
            // Next line is a hack.  EEPROM needs more time.
//            int timeout = (cmd[0] == 'U' || cmd[0] == 'W') ? 50 : 25;
            int timeout = 60;
            sport.serialWrite(cmd, timeout);
            response[0] = '\0';
            sport.serialRead(response);
            String numStr;
            switch ((char) response[0]) {
                case 'A': // Scale range.
                    powerMeter.setRange((char) response[1]);
                    break;
                case 'F': // Forward power.
                    numStr = new String(response, 1, 4);
                    pwrFwd = Double.valueOf(numStr);
                    break;
                case 'H': // Decay rate.
                    powerMeter.setDecayRate((char) response[1]);
                    break;
                case 'L': // LED state.
                    powerMeter.setLedsOn(response[2] == 'N');
                    break;
                case 'N': // Avg/PEP pwr on serial link.
                    powerMeter.setPEP(response[1] == 'P');
                    break;
                case 'P': // Peak vs Peak-Hold.
                    powerMeter.setPeakHold(response[2] == 'O');
                    break;
                case 'R': // Reflected power.
                    numStr = new String(response, 1, 4);
                    pwrRfl = Double.valueOf(numStr);
                    break;
                case 'S': // SWR.
                    numStr = new String(response, 1, 4);
                    swr = Double.valueOf(numStr);
                    // Set vu-meter to num
                    break;
                case 'U': // Current EEPROM settings.
                    parseState(response);
                    break;
                case 'V': // Firmware version.
                    fw = new String(response, 1, 4);
                    fw = "FW " + fw;
                    powerMeter.setFirmwareVersion(fw);
                    break;
                case 'W': // Write settings to EEPROM.
                    break;
                case 'X': // Current RAM settings.
                    parseState(response);
                    break;
                default:
                    // Do nothing.
                    break;
            }
        }
    }

    /**
     * Send a W1 command over the serial link and pause, typically 25 ms, to
     * allow W1 to respond. Response is then parsed and acted upon by setting
     * the software state to match state of the real W1. NB: for auto-range
     * mode, the updated range is chosen when new readings of both forward and
     * reflected power have been taken. Therefore, you'll want to take the
     * readings as close together as possible, e.g., with a w1cmd("FR") call.
     */
    public void w2cmd(String s) {
        if (s == null) {
            return;
        }
        byte cmd[] = new byte[1];
        byte[] response = new byte[13];
        for (int i = 0; i < s.length(); i++) {
            cmd[0] = (byte) s.charAt(i);
            // Next line is a hack.  EEPROM needs more time.
//            int timeout = (cmd[0] == 'U' || cmd[0] == 'W') ? 50 : 25;
            int timeout = 60; // 60;
            sport.serialWrite(cmd, timeout);
            for (int j = 0; j < response.length; j++)
                response[i] = '\0';
            sport.serialRead(response);
            String numStr;
            switch ((char) response[0]) {
                case 'A': // Toggle locking SWR Alarm On/Off.
                    powerMeter.setRange((char) response[1]);
                    break;
                case 'C': // Reset from SWR Alarm.
                    break;
                case 'F': // Forward power.
                    numStr = new String(response, 1, 8);
                    double mag = Double.valueOf(numStr.substring(0, 5));
                    int exp = Integer.valueOf(numStr.substring(6, 7));
                    pwrFwd = Double.valueOf(mag * Math.pow(10, -exp));
                    break;
                case 'H': // Decay rate.
                    powerMeter.setDecayRate((char) response[1]);
                    break;
                case 'L': // LED state.
                    powerMeter.setLedsOn(response[2] == 'N');
                    break;
                case 'N': // Avg/PEP pwr on serial link.
                    powerMeter.setPEP(response[1] == 'P');
                    break;
                case 'P': // Peak vs Peak-Hold.
                    powerMeter.setPeakHold(response[2] == 'O');
                    break;
                case 'R': // Reflected power.
                    numStr = new String(response, 1, 8);
                    mag = Double.valueOf(numStr.substring(0, 5));
                    exp = Integer.valueOf(numStr.substring(6, 7));
                    pwrRfl = Double.valueOf(mag * Math.pow(10, -exp));
                    break;
                case 'S': // SWR.
                    numStr = new String(response, 1, 4);
                    swr = Double.valueOf(numStr);
                    // Set vu-meter to num
                    break;
                case 'U': // Current EEPROM settings.
                    parseState(response);
                    break;
                case 'V': // Firmware version.
                    fw = new String(response, 1, 4);
                    fw = "FW " + fw;
                    powerMeter.setFirmwareVersion(fw);
                    break;
                case 'W': // Write settings to EEPROM.
                    break;
                case 'X': // Current RAM settings.
                    parseState(response);
                    break;
                default:
                    // Do nothing.
                    break;
            }
        }
    }

    /**
     * Parse and adjust update internal state so that it matches the state
     * described by the response to a "U" or "X" command. "U" returns settings
     * stored in EEPROM and "X" returns settings currently in use in RAM. From
     * the W1 manual, response to "U" and "X" commands is the same with the
     * exception of the first letter which matches the user command. E.g., in
     * response to a "U" the W1 replies:
     *
     * <p>
     * U<A|P><A|P><S|M|F><S|M|F>;
     *
     * <p>
     * where
     *
     * <ul> <li>First A or P is Average power or PEP for Forward Power LEDs.
     * <li>Second A or P is Average power or PEP for Forward or Reverse Power
     * serial data. <li>First S or M or F is Slow or Medium or Fast LED decay
     * Rate. <li>Second S or M or F is Slow or Medium of Fast Range drop rate.
     * </ul>
     */
    private void parseState(byte[] state) {
        // (State[2] is ignored because LED and serial data are
        // always set identically for fwd and rfl directions.)

        // Average or PEP power display.
        powerMeter.setPEP(state[1] == 'P');
        // LED decay rate.
        powerMeter.setDecayRate((char) state[3]);
        // Range drop rate.
        powerMeter.setDropRate((char) state[4]);
    }

    public double vswr(double fwd_W, double refl_W) {

        double gamma = Math.sqrt(refl_W / fwd_W);
        return (1 + gamma) / (1 - gamma);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        serialPortPopup = new javax.swing.JPopupMenu();
        bigPanel = new javax.swing.JPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setFocusable(false);

        bigPanel.setFocusable(false);
        bigPanel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                mouseHandler(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                mouseHandler(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                mouseHandler(evt);
            }
        });
        bigPanel.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                keyListener(evt);
            }
        });

        javax.swing.GroupLayout bigPanelLayout = new javax.swing.GroupLayout(bigPanel);
        bigPanel.setLayout(bigPanelLayout);
        bigPanelLayout.setHorizontalGroup(
            bigPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 768, Short.MAX_VALUE)
        );
        bigPanelLayout.setVerticalGroup(
            bigPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 533, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(bigPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(bigPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void keyListener(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_keyListener
        char key = evt.getKeyChar();
        System.out.println("HERE!");
        if (key != '\t') {
            return;
        }
        // Cycle between meter faces.
        curFace = (curFace + 1) % 2;
        switch (curFace) {
            case 0:
                powerMeter = analogPanel;
                remove(miniMeter);
                add(analogPanel);
                break;
            case 1:
                powerMeter = miniMeter;
                remove(analogPanel);
                add(miniMeter);
                break;
            case 2:
                break;
            default:
                break;
        }
        pack();
    }//GEN-LAST:event_keyListener

    private void mouseHandler(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mouseHandler
        if (evt.isPopupTrigger()) {
            serialPortPopup = mkPopUpMenu();
            serialPortPopup.show(this, evt.getX(), evt.getY());
        }
    }//GEN-LAST:event_mouseHandler

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /*
         * Set the Nimbus look and feel
         */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /*
         * If Nimbus (introduced in Java SE 6) is not available, stay with the
         * default look and feel. For details see
         * http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info
                    : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(W1.class.getName()).log(
                    java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(W1.class.getName()).log(
                    java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(W1.class.getName()).log(
                    java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(W1.class.getName()).log(
                    java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /*
         * Create and display the form
         */
        java.awt.EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
                new W1().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel bigPanel;
    private javax.swing.JPopupMenu serialPortPopup;
    // End of variables declaration//GEN-END:variables
}
